@@ -1,19 +1,60 @@
 import argparse, sys, logging, traceback
 
-from sssekai.entrypoint.apidecrypt import main_apidecrypt
-from sssekai.entrypoint.abdecrypt import main_abdecrypt
-from sssekai.entrypoint.rla2json import main_rla2json
-from sssekai.entrypoint.usmdemux import main_usmdemux
-from sssekai.entrypoint.abcache import main_abcache, DEFAULT_CACHE_DB_FILE
-from sssekai.entrypoint.abserve import main_abserve
-from sssekai.entrypoint.live2dextract import main_live2dextract
-from sssekai.entrypoint.spineextract import main_spineextract
-from sssekai.entrypoint.apphash import main_apphash
-from sssekai.entrypoint.il2cpp import main_il2cpp
-from sssekai.entrypoint.mvdata import main_mvdata
-from sssekai.entrypoint.moc3paths import main_moc3paths
-from sssekai.unity import sssekai_get_unity_version, sssekai_set_unity_version
+# region Deferred imports
+# This can be done a lot cleaner with importlib. But tools like PyInstaller really
+# won't like that.
 
+def main_apidecrypt(*args, **kwargs):
+    from sssekai.entrypoint.apidecrypt import main_apidecrypt
+    return main_apidecrypt(*args, **kwargs)
+
+def main_abdecrypt(*args, **kwargs):
+    from sssekai.entrypoint.abdecrypt import main_abdecrypt
+    return main_abdecrypt(*args, **kwargs)
+
+def main_rla2json(*args, **kwargs):
+    from sssekai.entrypoint.rla2json import main_rla2json
+    return main_rla2json(*args, **kwargs)
+
+def main_usmdemux(*args, **kwargs):
+    from sssekai.entrypoint.usmdemux import main_usmdemux
+    return main_usmdemux(*args, **kwargs)
+
+def main_abcache(*args, **kwargs):
+    from sssekai.entrypoint.abcache import main_abcache
+    return main_abcache(*args, **kwargs)
+
+def main_abserve(*args, **kwargs):
+    from sssekai.entrypoint.abserve import main_abserve
+    return main_abserve(*args, **kwargs)
+
+def main_live2dextract(*args, **kwargs):
+    from sssekai.entrypoint.live2dextract import main_live2dextract
+    return main_live2dextract(*args, **kwargs)
+
+def main_spineextract(*args, **kwargs):
+    from sssekai.entrypoint.spineextract import main_spineextract
+    return main_spineextract(*args, **kwargs)
+
+def main_apphash(*args, **kwargs):
+    from sssekai.entrypoint.apphash import main_apphash
+    return main_apphash(*args, **kwargs)
+
+def main_il2cpp(*args, **kwargs):
+    from sssekai.entrypoint.il2cpp import main_il2cpp
+    return main_il2cpp(*args, **kwargs)
+
+def main_mvdata(*args, **kwargs):
+    from sssekai.entrypoint.mvdata import main_mvdata
+    return main_mvdata(*args, **kwargs)
+
+def main_moc3paths(*args, **kwargs):
+    from sssekai.entrypoint.moc3paths import main_moc3paths
+    return main_moc3paths(*args, **kwargs)
+
+# endregion
+
+from sssekai.unity import sssekai_get_unity_version, sssekai_set_unity_version
 from sssekai.fmt.rla import RLA_VERSIONS
 
 logger = logging.getLogger(__name__)
@@ -98,7 +139,7 @@ This crypto applies to:
         "infile", type=str, help="input file", **gooey_only(widget="FileChooser")
     )
     usmdemux_parser.add_argument(
-        "outdir", type=str, help="output directory", **gooey_only(widget="DirChooser")
+        "outfile", type=str, help="output file where the .USM file will be saved", **gooey_only(widget="FileSaver")
     )
     usmdemux_parser.set_defaults(func=main_usmdemux)
     # abcache
@@ -116,7 +157,7 @@ This crypto applies to:
         "--db",
         type=str,
         help="""cache database file path (default: %(default)s)""",
-        default=DEFAULT_CACHE_DB_FILE,
+        default="~/.sssekai/abcache.db",
         **gooey_only(widget="FileChooser"),
     )
     group.add_argument(
@@ -124,6 +165,13 @@ This crypto applies to:
         action="store_true",
         help="skip all metadata updates and use cached ones as is. this supercedes 'game version options' and 'authentication options'",
     )
+    group.add_argument(
+        "--dump",
+        type=str,
+        help="dump the AssetBundle index to the specified JSON file and exit",
+        default=None,
+        **gooey_only(widget="FileSaver"),
+    )
     group = abcache_parser.add_argument_group(
         "game version options",
         "NOTE: these options are used to *update* the cache database. Use --no-update to skip updating. Also, you may want to read the Wiki to know how to get these values.",
@@ -259,7 +307,7 @@ This crypto applies to:
         "--db",
         type=str,
         help="""cache database file path (default: %(default)s)""",
-        default=DEFAULT_CACHE_DB_FILE,
+        default="~/.sssekai/abcache.db",
         **gooey_only(widget="FileChooser"),
     )
     abserve_parser.add_argument(
