@@ -256,6 +256,13 @@ class SSSekaiDatabase:
     sekai_system_data: SekaiSystemData = None
     sekai_gameversion_data: SekaiGameVersionData = None
 
+    def reset(self):
+        self.sekai_user_data = None
+        self.sekai_user_auth_data = None
+        self.sekai_abcache_index = None
+        self.sekai_system_data = None
+        self.sekai_gameversion_data = None
+
 
 class AbCacheBundleNotFoundError(Exception):
     bundleName: str
@@ -284,7 +291,7 @@ class AbCache(Session):
                     return "https://%s" % url
                 return "https://production-game-api.sekai.colorfulpalette.org"
             case "en":
-                return "https://n-production-game-api.sekai-en.com"
+                return f"https://n-{self.SEKAI_API_PROFILE}-game-api.sekai-en.com"
             case "tw":
                 return "https://mk-zian-obt-cdn.bytedgame.com"
             case "kr":
@@ -356,7 +363,8 @@ class AbCache(Session):
             case "jp":
                 return f"https://{self.SEKAI_API_PROFILE}-{self.SEKAI_AB_HOST_HASH}-assetbundle-info.sekai.colorfulpalette.org/api/version/{self.SEKAI_ASSET_VERSION}/{self.SEKAI_AB_HASH}/os/{self.config.app_platform}"
             case "en":
-                return f"https://assetbundle-info.sekai-en.com/api/version/{self.SEKAI_ASSET_VERSION}/os/{self.config.app_platform}"
+                # `n` for what, exactly?                
+                return f"https://n-{self.SEKAI_API_PROFILE}-{self.SEKAI_AB_HOST_HASH}-assetbundle-info.sekai-en.com/api/version/{self.SEKAI_ASSET_VERSION}/os/{self.config.app_platform}"
         if self.config.app_region in REGION_ROW:
             return f"{self.SEKAI_AB_ROW_CDN}/AssetBundle/{self.SEKAI_AB_ROW_VERSION}/Release/{self.SEKAI_AB_ROW_PATH}/{self.SEKAI_APP_PLATFORM}{self.SEKAI_AB_ROW_VERSION_NUMBER}/AssetBundleInfoNew.json"
         else:
@@ -368,22 +376,25 @@ class AbCache(Session):
             case "jp":
                 return f"https://{self.SEKAI_API_PROFILE}-{self.SEKAI_AB_HOST_HASH}-assetbundle.sekai.colorfulpalette.org/"
             case "en":
-                return f"https://assetbundle.sekai-en.com/"
+                return f"https://n-{self.SEKAI_API_PROFILE}-{self.SEKAI_AB_HOST_HASH}-assetbundle.sekai-en.com/"
         if self.config.app_region in REGION_ROW:
             return f"{self.SEKAI_AB_ROW_CDN}/AssetBundle/{self.SEKAI_AB_ROW_VERSION}/Release/{self.SEKAI_AB_ROW_PATH}/"
         else:
             raise NotImplementedError
 
     def _update_request_headers(self):
+        app_platform_names = {
+            'ios':'iOS', 'android':'Android'
+        }
         self.headers.update(
             {
                 "Accept": "application/octet-stream",
                 "Content-Type": "application/octet-stream",
                 "Accept-Encoding": "deflate, gzip",
                 "User-Agent": "UnityPlayer/%s" % sssekai_get_unity_version(),
-                "X-Platform": self.config.app_platform.capitalize(),
+                "X-Platform": app_platform_names.get(self.config.app_platform,self.config.app_platform),
                 "X-DeviceModel": "sssekai/%s" % __version__,
-                "X-OperatingSystem": self.config.app_platform.capitalize(),
+                "X-OperatingSystem": app_platform_names.get(self.config.app_platform,self.config.app_platform),
                 "X-Unity-Version": sssekai_get_unity_version(),
                 "X-App-Version": self.SEKAI_APP_VERSION,
                 "X-App-Hash": self.SEKAI_APP_HASH,
@@ -488,7 +499,7 @@ class AbCache(Session):
         return f"{self.SEKAI_API_ENDPOINT}/api/suite/user/{self.SEKAI_USERID}"
 
     @property
-    def SEKAI_API_MASTER_SUITE_URLS(self):
+    def SEKAI_API_MASTER_SUITE_URLS(self) -> List[str]:
         match self.config.app_region:
             case "jp":
                 self.raise_for_auth()
@@ -504,7 +515,7 @@ class AbCache(Session):
                 ]
         if self.config.app_region in REGION_ROW:
             self.raise_for_auth()
-            return f"{self.SEKAI_AB_ROW_CDN}/{self.SEKAI_AB_ROW_PATH}/master-data-{self.database.sekai_user_auth_data.cdnVersion}.info"
+            return [f"{self.SEKAI_AB_ROW_CDN}/{self.SEKAI_AB_ROW_PATH}/master-data-{self.database.sekai_user_auth_data.cdnVersion}.info"]
         else:
             raise NotImplementedError
 
@@ -537,7 +548,23 @@ class AbCache(Session):
         """
         self._update_request_headers()
         if data is not None:
-            data = packb(data)
+            # XXX: packb() does not cover all cases.
+            #
+            # Python msgpack implementation doesn't really support variadic float precision within a single pack.
+            # This generally shouldn't be an issue if the upstream unpacking routine is type-agnostic since
+            # the type is described in the packed binary as per the spec.
+            #
+            # However, if the upstream implementation is not type-agnostic, it WILL lead to issues.
+            # While most respect the type opcodes (e.g. msgpack-c, msgpack-python, msgpack in Golang), some implementations (e.g. MessagePack-CSharp) disregards
+            # them altogether when concrete (C#) types are provided. And will raise an exception in such cases when a double is encoded in the place of a float.
+            # See also:
+            #   https://github.com/mos9527/sssekai/issues/47#issuecomment-3150827990
+            #
+            # Thus `use_single_float=True` is used to cover and only cover single precision cases. 
+            # In the *current* API seen in the game binary, this is fine. And in AbCache's case floats are not at all used.
+            # But this may change in the future, in which case consult the following issues:
+            #   https://github.com/msgpack/msgpack-python/issues/326
+            data = packb(data, use_single_float=True)
             data = encrypt(data, SEKAI_APIMANAGER_KEYSETS[self.config.app_region])
         resp = self.request(method=method, url=url, data=data, **kwargs)
         if 400 <= resp.status_code < 600:
@@ -690,6 +717,7 @@ class AbCache(Session):
     def save(self, f: BinaryIO):
         self._update_request_headers()
         self.database.cached_headers = self.headers
+        self.database.config.version = __version_tuple__
         logger.info("Saving cache: %s" % self)
         dump(self.database, f)
 
