@@ -4,7 +4,7 @@ from logging import getLogger
 logger = getLogger(__name__)
 
 
-def sega_register_anonymous_user(cache: AbCache) -> SekaiUserData:
+def register_as_anonymous_user_sega(cache: AbCache) -> SekaiUserData:
     """Registers an anonymous account on SEGA servers, and writes relavant data to the AbCacheobject.
 
     Args:
@@ -14,7 +14,9 @@ def sega_register_anonymous_user(cache: AbCache) -> SekaiUserData:
         SekaiUserData: The user data object.
     """
     logger.info("Registering user data")
-    cache._update_signatures()
+    # https://github.com/mos9527/sssekai/issues/31
+    cache._update_request_headers()
+    cache.update_signatures()
     payload = {
         "platform": cache.headers["X-Platform"],
         "deviceModel": cache.headers["X-DeviceModel"],
