@@ -2,9 +2,12 @@ from base64 import b64decode, b64encode
 from struct import unpack as s_unpack
 from io import BytesIO
 from collections import defaultdict
+from typing import Generator, Tuple, TypeVar, List
 import math, gzip
 import msgpack
 
+RLA_VERSIONS = [(1, 0), (1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (1, 6), (1, 7)]
+
 read_int = lambda stream, nbytes, signed=False: int.from_bytes(
     stream.read(nbytes), "little", signed=signed
 )
@@ -21,6 +24,14 @@ class SSEDataLengthOutOfRangeException(Exception):
         super().__init__(f"Needed {needed} bytes, but only {current} bytes available.")
 
 
+class SSEMissingSplitPacketException(Exception):
+    missing: List[Tuple[object, int]]  # (timeStamp, splitId)
+
+    def __init__(self, missing):
+        self.missing = missing
+        super().__init__(f"Missing %d split packet(s): %s" % (len(missing), missing))
+
+
 # Sekai_Streaming_StreamingCommon__CheckHeader
 def decode_buffer_base64(buffer: bytes) -> tuple[int, bytes]:
     """Decodes the 'RTVL' SSE Message payload into a header signature and data.
@@ -32,7 +43,7 @@ def decode_buffer_base64(buffer: bytes) -> tuple[int, bytes]:
         SSEDataLengthOutOfRangeException: If the buffer length does not match the expected length.
 
     Returns:
-        (int, bytes): Header Signature and decoded data payload
+        ((splitId, splitIndex, splitNum, dataLength, totalDataLength, base64) or None, decoder signature, data in decoded bytes)
     """
     # They really want you to believe it's Base64...
     # [4 bytes: RTVL][6 bytes : length in hex][Base64 T/F][Split T/F][3 bytes: Signature][data, Base64]
@@ -52,9 +63,33 @@ def decode_buffer_base64(buffer: bytes) -> tuple[int, bytes]:
     current_length = len(data) + 15
     if current_length != encoded_length:
         raise SSEDataLengthOutOfRangeException(encoded_length, current_length)
-    if is_base64_encoded:
-        data = b64decode(data)
-    return header_signature, data
+    if is_split:
+        __sizes_pre = (0, 5, 10, 15, 25, 35)
+        splitInfo = data[: __sizes_pre[-1]]
+        splitId, splitIndex, splitNum, dataLength, totalDataLength = map(
+            int,
+            (
+                splitInfo[__sizes_pre[i] : __sizes_pre[i + 1]].decode()
+                for i in range(len(__sizes_pre) - 1)
+            ),
+        )
+        data = data[__sizes_pre[-1] :]
+        return (
+            (
+                splitId,
+                splitIndex,
+                splitNum,
+                dataLength,
+                totalDataLength,
+                is_base64_encoded,
+            ),
+            header_signature,
+            data,
+        )
+    else:
+        if is_base64_encoded:
+            data = b64decode(data)
+        return None, header_signature, data
 
 
 # Sekai_Streaming_SubscribeDecoder__Deserialize
@@ -172,6 +207,12 @@ def decode_streaming_data(
         get_next_ushort() * 0.01,
         get_next_ushort() * 0.01,
     )  # ReadUShortVector3
+    get_next_quaternion = lambda: (
+        get_next_float(),
+        get_next_float(),
+        get_next_float(),
+        get_next_float(),
+    )  # ReadQuaternion
     get_next_tiny_int = lambda type: {
         "+": lambda: read_int(stream, 2),
         "*": lambda: read_int(stream, 1, True),
@@ -263,7 +304,7 @@ def decode_streaming_data(
                     "channels": channels,
                     "sampleRate": sample_rate,
                     "encoding": "raw",
-                    "data": b64encode(data).decode(),
+                    "data": data,
                 }
             else:
                 data = stream.read(data_length)  # HCA w/o metadata
@@ -272,7 +313,7 @@ def decode_streaming_data(
                     "channels": channels,
                     "sampleRate": sample_rate,
                     "encoding": "hca",
-                    "data": b64encode(data).decode(),
+                    "data": data,
                 }
         case 3:
             # Sekai_Streaming_StatusData
@@ -316,9 +357,25 @@ def decode_streaming_data(
                     if version >= (1, 5)
                     else {}
                 ),
+                **(
+                    {
+                        "eyeLookAtUvLimit": get_next_quaternion(),
+                    }
+                    if version >= (1, 6)
+                    else {}
+                ),
+                **(
+                    {
+                        "preloadReserveFirstCharacterDelay": get_next_float(),
+                        "preloadReserveCharacterInterval": get_next_float(),
+                    }
+                    if version >= (1, 7)
+                    else {}
+                ),
             }
             read_character_status = lambda: {
                 "costumeIndex": get_next_int(),
+                **({"visible": get_next_mask()} if version >= (1, 6) else {}),
                 **({"useFx": get_next_mask()} if version >= (1, 2) else {}),
                 "timeStamp": get_next_long(),
             }
@@ -363,61 +420,97 @@ def decode_streaming_data(
 
 result = defaultdict(dict)
 
+T = TypeVar("T")
 
-def read_rla_frame(buffer: bytes, version=(1, 0), strict=True) -> dict:
-    """Parses a single frame of the Sekai RLA file format used in 'streaming_live/archive' assets.
+
+def read_rla_frames(
+    reader: Generator[Tuple[T, bytes], None, None], version=(1, 0), strict=True
+) -> Generator[Tuple[T, dict], None, None]:
+    """Parses a stream of rla fragments and automatically processes split data
+
+    There's NO guarantee that the frames are sorted by the frame ticks.
 
     Args:
-        buffer (bytes): Frame buffer
-        version (tuple, optional): RLA version, found in respective RLH (JSON) header files. range: (1,0) to (1,5). Defaults to (1,0).
+        reader (Generator[Tuple[T, bytes], None, None]): A generator that yields a tuple of timestamp and buffer data
+        version (tuple, optional): RLA version, found in respective RLH (JSON) header files. range: see RLA_VERSIONS. Defaults to (1,0).
         strict (bool, optional): If False, incomplete packets will be returned as is. Defaults to True.
 
-    Returns:
-        dict: Parsed frame data
+    Yields:
+        Generator[Tuple[T, dict], None, None]: A generator that yields a tuple of timestamp and parsed frame data. Timestamp format is provided by the reader.
     """
-    header_signature, data = decode_buffer_base64(buffer)
-    decoder_signature, data = decode_buffer_payload(data)
     assert (
-        header_signature == decoder_signature
-    ), "mismatching signature (header/decoder). packet may be corrupt"
-    payload = decode_streaming_data(version, decoder_signature, data, strict)
-    return payload
+        version >= RLA_VERSIONS[0] and version <= RLA_VERSIONS[-1]
+    ), "unsupported version"
+    __buffers = defaultdict(
+        list
+    )  # splitIndex:([timestamp, splitInfo, header_signature, data])
+
+    def decode_buffer(header_signature, data):
+        decoder_signature, data = decode_buffer_payload(data)
+        assert (
+            header_signature == decoder_signature
+        ), "mismatching signature (header/decoder). packet may be corrupt"
+        payload = decode_streaming_data(version, decoder_signature, data, strict)
+        return payload
+
+    for timestamp, buffer in reader:
+        try:
+            split_info, header_signature, data = decode_buffer_base64(buffer)
+        except Exception as e:
+            if strict:
+                raise e
+            # Otherwise fail silently
+        if split_info:
+            splitId, splitIndex, splitNum, dataLength, totalDataLength, base64 = (
+                split_info
+            )
+            __buffers[splitId].append((timestamp, split_info, header_signature, data))
+            if len(__buffers[splitId]) == splitNum:
+                buffer = b"".join(
+                    data
+                    for _, _, _, data in sorted(
+                        __buffers[splitId], key=lambda x: x[1][1]
+                    )  # splitIndex
+                )
+                if strict:
+                    assert len(buffer) == totalDataLength, "incomplete split packet"
+                del __buffers[splitId]
+                if base64:
+                    buffer = b64decode(buffer)
+                yield timestamp, decode_buffer(header_signature, buffer)
+        else:
+            yield timestamp, decode_buffer(header_signature, data)
+
+    if __buffers and strict:
+        missing = [(payload[0][0], splitId) for splitId, payload in __buffers.items()]
+        raise SSEMissingSplitPacketException(missing)
 
 
-def read_rla(src: BytesIO, version=(1, 0), strict=True) -> dict:
+def read_archive_rla_frames(
+    src: BytesIO, version=(1, 0), strict=True
+) -> Generator[Tuple[int, dict], None, None]:
     """Parses the Sekai RLA file format used in 'streaming_live/archive' assets.
 
+    The frames are sorted by the frame ticks.
+
     Args:
         src (BytesIO): Source RLA file stream
-        version (tuple, optional): RLA version, found in respective RLH (JSON) header files. range: (1,0) to (1,5). Defaults to (1,0).
+        version (tuple, optional): RLA version, found in respective RLH (JSON) header files. range: see RLA_VERSIONS. Defaults to (1,0).
         strict (bool, optional): If False, incomplete packets will be returned as is. Defaults to True.
 
-    Returns:
-        dict: Parsed RLA data. The dictionary is sorted by the frame ticks.
+    Yields:
+        Generator[Tuple[int, dict], None, None]: A generator that yields a tuple of timestamp and parsed frame data
     """
-    result = defaultdict(dict)
 
-    def read_frames():
-        ticks = read_int(src, 8)
-        if ticks:
-            buffer_length = read_int(src, 4)
-            buffer = src.read(buffer_length)
-            payload = read_rla_frame(buffer, version, strict)
-            result[ticks].setdefault(payload["type"], list()).append(payload)
-            return True
-        return False
+    def __packet_gen():
+        nonlocal tick
 
-    while read_frames():
-        pass
-    result = dict(sorted(result.items(), key=lambda x: x[0]))
-    return result
+        while tick := read_int(src, 8):
+            if tick:
+                buffer_length = read_int(src, 4)
+                buffer = src.read(buffer_length)
+                yield tick, buffer
 
-
-if __name__ == "__main__":
-    import sys
-    from timeit import timeit
-
-    fp = open(sys.argv[-1], "rb")
-    buffer = fp.read()
-    payload = read_rla_frame(buffer)
-    pass
+    packets = __packet_gen()
+    for tick, frame in read_rla_frames(packets, version, strict):
+        yield tick, frame
