@@ -243,9 +243,12 @@ class KeyframeHelper:
     def interpolate(
         t, lhs: "KeyframeHelper", rhs: "KeyframeHelper", unit_t: bool = False
     ) -> float | Vector3 | Quaternion:
+        EPS = 1e-8
         if not rhs:
             return lhs.value
-        dx = rhs.time - lhs.time
+        dx = rhs.time - lhs.time        
+        if dx < EPS:
+            return lhs.value
         if not unit_t:
             t -= lhs.time
             t /= dx  # Normalized to [0,1]
@@ -348,7 +351,7 @@ class CurveHelper:
         rhs = lhs.next
         return KeyframeHelper.interpolate(t, lhs, rhs)
 
-    def resample_dense(self, times: List[float]) -> "CurveHelper": # O(n log n). Can be done linearly though.
+    def resample_dense(self, times: List[float]) -> "CurveHelper": # O(n)
         """Resamples the curve at given times **as a Dense curve**, losing all tangent information.
 
         This guarantees accuracy when played back at a constant sample rate at the cost of space
@@ -360,7 +363,13 @@ class CurveHelper:
         Returns:
             CurveHelper: A new CurveHelper with the sampled values
         """
-        data = [self.evaluate(t) for t in times]
+        j = 0
+        data = [None] * len(times)
+        for i, t in enumerate(times):
+            while j + 1 < len(self.Data) and self.Data[j].time <= t:
+                j += 1            
+            lhs = max(j - 1, 0)            
+            data[i] = KeyframeHelper.interpolate(t, self.Data[lhs], self.Data[lhs].next)
         slope = from_floats(*([0] * num_floats(data[0])))
         data = [KeyframeHelper(t, self.Binding.typeID, value, isDense=True, inSlope=slope, outSlope=slope) for t, value in zip(times, data)]
         for i in range(1, len(data)):
