@@ -1,11 +1,11 @@
 from io import BytesIO
 from sssekai.crypto.AssetBundle import decrypt_iter
 
-from . import UnityPy, sssekai_get_unity_version
-from UnityPy import Environment
+from . import sssekai_get_unity_version
+import UnityPy
 
 
-def load_assetbundle(file: BytesIO) -> Environment:
+def load_assetbundle(file: BytesIO) -> UnityPy.Environment:
     UnityPy.config.FALLBACK_UNITY_VERSION = sssekai_get_unity_version()
     stream = BytesIO()
     for block in decrypt_iter(lambda nbytes: file.read(nbytes)):
