@@ -1,4 +1,4 @@
-import UnityPy, logging
+import logging
 
 logger = logging.getLogger(__name__)
 
@@ -17,10 +17,3 @@ def sssekai_set_unity_version(value):
             f"Setting Unity Version from {_sssekai_unity_version} to {value}"
         )
         _sssekai_unity_version = value
-
-
-# Monkey-patching modules
-from . import AnimationClip
-
-# Other modules
-from . import AssetBundle
